#
#     26 Oct 96   for da adopted from charissa version
#    8 jan 98   modified for use with F2VB module


set frametitle "MIDAS Data Acquisition System Control (CLARA/PRISMA)"


set VERSION "PRISMA"

set env(RDOUThdwre) D2VB

set VMEPresent 1
set VXIPresent 1

set VMEEnable 1
set VXIEnable 1

set RDOUTEnable 1
set PRISMAEnable 1

set histEnable 1
set HistMode 1

set txEnable 1

set MaxTries 100

set TSPresent 1

set CheckDAPeriod 10

loadsource  sys.common


set TestBuffer "01f80080  11f80001  00010080  01010100  02010180  03010200  04010280  31f80000   \
     0ff80080  1ff80001  00010080  01010100  02010180  03010200  04010280  3ff80000"

set SetupMessage "\n\
Are you sure you wish to SETUP the data acquisition system?\n\n\
SETUP will STOP data acquisition.\n\
Configure the VXI Control software and restore VXI parameter settings.\n\
This may take several minutes.\n\n\
If you decide to continue please wait for the SETUP sequence to complete and DO NOT touch the system while it is in progress."


proc checkinstallation  {} {
   global OS VERSION env

   global RDOUTEnable; if {$RDOUTEnable == 0} then {return 0}

   set ID [read-register EXEC.ID]
   set OS [first $ID]
   set version  [second $ID]
   set release [third $ID]

   if {$version != "D2VB"} then {
       set msg " WARNING!  Possible system installation or setup error! \n\
       Require Data Acquisition program $VERSION but discovered \n\
       OS $OS; Program $version; Release $release\n\
       Do you wish to continue?"
       if {[midas-confirmation $msg] == 0} then {return 1}
   }

   set env(RTOS) $OS
   return 0
}

proc Custom_Start {} {starttask titris; starttask xdaq}
proc Custom_Redisplay {} {}

proc Custom_PreSetup {opt} {

#    create TITRIS registers

    inform titris setup

#    setup hardware configuration information needed by the DA program

    set crates [inform vre inquire chanlist]
    write-register NumVXICrates [llength $crates]

    set cratemap 0
    foreach c $crates {set cratemap [expr $cratemap + (1 << $c)]}
    write-register CrateMap $cratemap

    write-register TitrisPresent 1
}

proc Custom_PostSetup {opt} {

#    setup the Titris system

    inform titris start

    inform titris select_module C
    inform titris click-slave            ;#    CLARA Titris module now in Slave mode  (ie reset)

    inform titris select_module C
    inform titris click-master           ;#    CLARA Titris module now in Master mode

    click-prisma 0 2                     ;#   send SOAP Halt command to the Prisma applications followed by Configure command

    inform titris select_module C
    inform titris click-go               ;#    CLARA Titris module now in GO state

    inform titris close-frame
}

proc Custom_PreStop {} {click-prisma 0 0}
proc Custom_PostGo {} {click-prisma 0 1}

proc Custom_PreGo {} {inform vre vre-reloadDSP VXI}

proc delete_spectra {} {
   global SpectrumServer
}

proc create_spectra {} {
    global SpectrumServer

    set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 8192 :name 'Stat' !new)" } m]
    set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 8192 :name 'Rate' !new)" } m]
    set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 65536 :name 'Time' !new)" } m]

#     all other spectra created by the VXI software histogram interface
    map_spectra
}

proc map_spectra {} {
   global EGreport RDOUThdwre

   set EGreport_SAVE $EGreport
   set EGreport -1

   set l [spectra-names]
   foreach item $l {
       set name [second $item]
       if {"$name" == "Rate"} continue
       map_spectrum $name
   }

   write-register $RDOUThdwre.Cspectramapped 1

   set EGreport $EGreport_SAVE

   set-footer "mapped [llength $l] spectra"
}

proc map_needed {} {
    global RDOUThdwre
    write-register $RDOUThdwre.Cspectramapped 0
}

proc map_spectrum {s} {

   global RDOUTEnable; if {$RDOUTEnable == 0} then {return}

   execute-command 10 "Mapping spectrum $s" "$s"
}

proc click-prismaenable {w vl} {
    global PRISMAEnable
    set PRISMAEnable $vl
    
    redisplay-prisma
}

proc click-prisma  {w vl} {
   global PRISMAEnable
   global BUstate RMstate RUIstate

   if {$PRISMAEnable == 0} then {return}
   
   set-footer "Performing action on XDAQ (Prisma) applications"
   
   redisplay-prisma  ;#  will call Condconfigureallhosts
   inform xdaq Condconfigureallapplications

    set z [catch {
        if {$vl==0} then {
#    Halt
              inform xdaq disableapplications
	      if {[prismaState] != 0} then {
	          midas-warning "Not all Prisma (XDAQ) applications are in the correct (Ready) state after HALT.\n\
		       BU is $BUstate; RM is $RMstate; RUI is $RUIstate"
	      }
        }
        if {$vl==1} then {
#    GO

              if {[prismaState] != 0} then {
	          midas-warning "Not all Prisma (XDAQ) applications are in the correct (Ready) state. Try a SETUP. \n\
 	               BU is $BUstate; RM is $RMstate; RUI is $RUIstate"
	      } else {
                  inform xdaq enableapplications
		  if {[prismaState] != 1} then {
	              midas-warning "Not all Prisma (XDAQ) applications are in the correct (Enabled) state after GO.\n\
		      	  BU is $BUstate; RM is $RMstate; RUI is $RUIstate"
		  }
	      }
        }
        if {$vl==2} then {
#    Setup
              inform xdaq do_applications Halt
              inform xdaq do_applications Configure
	      if {[prismaState] != 0} then {
	          midas-warning "Not all Prisma (XDAQ) applications are in the correct (Ready) state after SETUP.\n\
		  	  BU is $BUstate; RM is $RMstate; RUI is $RUIstate"
              }
        }
        if {$vl==3} then {
#    Reset 
        }
    } m ]
    redisplay-prisma
    clear-footer
}

proc prismaState {} {
    global BUstate RMstate RUIstate
    
   inform xdaq Condconfigureallhosts
   
   set BUstate [inform xdaq read_parameter 1 15 stateName]
   switch $BUstate {
      Halted   {set bustate 10}
      Enabled  {set bustate 1}
      Ready    {set bustate 0}
      default  {set bustate 100}
   }
   
   set RMstate [inform xdaq read_parameter 0 11 stateName]
   switch $RMstate {
      Halted   {set rmstate 10}
      Enabled  {set rmstate 1}
      Ready    {set rmstate 0}
      default  {set rmstate 100}
   }
   
   set RUIstate [inform xdaq read_parameter 0 12 stateName]
   switch $RUIstate {
      Halted   {set ruistate 10}
      Enabled  {set ruistate 1}
      Ready    {set ruistate 0}
      default  {set ruistate 100}
   }
   
   set state [expr $bustate + $rmstate + $ruistate]
   if {$state == 0} {return 0}
   if {$state == 3} {return 1}
   if {$state < 3} {return -1}
   if {$state < 100} {return -2}
   return -3
}

proc redisplay-prisma {} {
   global PRISMAEnable
   global BUstate RMstate RUIstate
   if {$PRISMAEnable == 0} then {:modify PRISMA :value -1; return}
   
   set state [prismaState]
   
   :modify PRISMA :value $state
   :modify PWARN :label ""
   if {$state == -1 || $state == -2} {:modify PWARN :label "Warning"}
   if {$state == -3} {:modify PWARN :label "Error - Try SETUP"}
   if {$state < 0} {insert-log "XDAQ applications: BU is $BUstate; RM is $RMstate; RUI is $RUIstate"}
}

proc reset-prisma {} {
    global PRISMAEnable
    
    if {$PRISMAEnable == 0} then {return}

    set r [midas-confirmation "You are about to RESET the Prisma (XDAQ) applications.\n\
              This will reboot the VME (VxWorks) system and restart the Merge (Linux) application.\n\
	      If these actions fail you will have to manually fix the problem!\n\n\
	      Are you sure you want to do this?" "Yes" "No"]
    if {$r != 0} {return}
    
    inform xdaq resetallhosts
    
    midas-warning "All Prisma (XDAQ) services have been reset.\n\
          Wait about 15 seconds and then click the Update button to check the application states\n\
	  As long as all 3 applications are in the Halted state (see session log) then you should SETUP"
}

proc update-prisma {} {
    global PRISMAEnable
    global BUstate RMstate RUIstate
    
    if {$PRISMAEnable == 0} then {return}
    
     redisplay-prisma
     insert-log "XDAQ applications: BU is $BUstate; RM is $RMstate; RUI is $RUIstate"
}

proc BUstats {} {
    global PRISMAEnable
    
    if {$PRISMAEnable == 0} then {return [list 0 0]}
    
    inform xdaq read_parameter 1 15 stateName
    set data [inform xdaq do_currentapplication Query]
    
    return [list [lindex $data 4] [lindex $data 5]]
}


